#include "DEventMap.hpp"
#include "DBuffer.hpp"

using namespace std;
using namespace libdasyne;

DEventMap::DEventMap(uint8_t newMsgType)
{
    msgType = newMsgType;
    haveSingle = false;
}//constructor

DEventMap::~DEventMap(void)
{
}//destructor

void DEventMap::addHandler(boost::function<void (DBuffer&)> newHandler)
{
    if (handlers.size() == 0)
    {
        haveSingle = true;
        singleHandler = newHandler;
    }//if size()
    else
    {
        DEngine::addFatal("Tried to add a handler for message type ");
        DEngine::addFatal(DEngine::intToStr(msgType));
        DEngine::fatalError(" without giving a subtype, but there are handlers for subtypes defined");
    }//else size()
}//addHandler

void DEventMap::addHandler(uint16_t subType,
                           boost::function<void (DBuffer&)> newHandler)
{
    map<uint16_t, boost::function<void (DBuffer&)> >::iterator mapIter;//points to an element in handlers

    if (haveSingle)
    {
        DEngine::addFatal("Tried to add a handler for message type ");
        DEngine::addFatal(DEngine::intToStr(msgType));
        DEngine::addFatal(", subtype ");
        DEngine::addFatal(DEngine::intToStr(subType));
        DEngine::fatalError(", but it already has a handler with no subtype");
    }//if haveSingle

    mapIter = handlers.find(subType);
    if (mapIter == handlers.end())
        handlers.insert(pair<uint16_t, boost::function<void (DBuffer&)> >(subType, newHandler));
    else
        mapIter->second = newHandler;
}//addHandler

bool DEventMap::getHandler(uint16_t subType,
                           boost::function<void (DBuffer&)> &theHandler)
{
    bool result = true;//false if we couldn't find a handler
    map<uint16_t, boost::function<void (DBuffer&)> >::iterator mapIter;//points to an element in handlers

    mapIter = handlers.find(subType);
    if (mapIter == handlers.end())
        result = false;
    else
        theHandler = mapIter->second;
        
    return result;
}//getHandler

unsigned int DEventMap::size(void)
{
    return handlers.size();
}//size()

bool DEventMap::getHaveSingle(void)
{
    return haveSingle;
}//getHaveSingle

boost::function<void (DBuffer&)> DEventMap::getSingleHandler(void)
{
    if (!haveSingle)
        DEngine::fatalError("DEventMap::getSingleHandler(): no single handler defined!");
        
    return singleHandler;
}//getSingleHandler
