/*
 * Copyright (C) 2011 David C. McCallum
 * For conditions of distribution and use, see the copyright notice in Dasyne.hpp
*/

/* DESCRIPTION
   Accepts TCP connections and spawns a DClient for each one.
*/

#ifndef DSERVER_HPP_INCLUDED
#define DSERVER_HPP_INCLUDED

#include "DEngine.hpp"

namespace libdasyne
{
    //The current state of the listen socket
    enum ListenerState
    {
        DA_UNBOUND, DA_BOUND, DA_LISTENING
    };//ListenerState

    class DClient;
    class DEngine;

    class DServer
    {
        public:
            //constructor
            DServer(DEngine *newNetEngine);

            //destructor
            virtual ~DServer(void);

            //Bind the socket to the specified port number, returns false on failure and
            //sets errMsg
            bool bind(libdasyne::IpVersion ipVersion,
                      uint16_t portNumber);

            //Start listening for new connections, returns false on failure and sets
            //errMsg. newHandler will be called whenever a client connects. The event handler takes
            //ownership of the socket, and should delete it when finished.
            bool startListening(boost::function<void (boost::asio::ip::tcp::socket*)> newHandler);

            //Closes the socket and stops listening. To accept additional connections, bind() and
            //startListening() will need to be called.
            void close(void);

            //Event handler, called when there's an error accepting a connection
            void setErrorHandler(boost::function<void (std::string)> newHandler);

            //Returns a description of the most recent error
            std::string getErrMsg(void);

        protected:
            //Handles events for this class
            DEngine *netEngine;

            //A description of the most recent error
            std::string errMsg;

        private:
            //The current state of the listening socket
            libdasyne::ListenerState myState;

            //Accepts new connections
            boost::asio::ip::tcp::acceptor *listenSocket;

            //A new connection to a client
            boost::asio::ip::tcp::socket *clientSocket;

            //The new client's address and port number
            boost::asio::ip::tcp::acceptor::endpoint_type clientAddress;

            //Event handlers
            boost::function<void (std::string)> errorHandler;
            boost::function<void (boost::asio::ip::tcp::socket*)> connectHandler;

            //True if the error event hander has been set
            bool haveErrorHandler;

            //############### Functions ###############

            //Call connectHandler when a client connects
            void getNextConnection(void);

            //Event handler, called when a client connects
            void connectFinished(const boost::system::error_code &errorCode);

    };//DServer
}//namespace libdasyne
#endif
